from healdata_utils.schemas import convert_frictionless_to_jsonschema
from pathlib import Path
import yaml


def test_convert_frictionless_to_jsonschema():
    with Path(__file__).parent.joinpath("data/test_frictionless_schema.yaml").open(
        mode="r"
    ) as f:
        frictionless_schema = yaml.safe_load(f)
    jsonschema_props = convert_frictionless_to_jsonschema(frictionless_schema)
    assert jsonschema_props == {
        "type": "object",
        "required": ["name", "description"],
        "properties": {
            "module": {
                "oneOf": [
                    {
                        "description": "Module (a place to put the section, form, or other broad category used \nto group variables.\n",
                        "title": "Module (i.e., section,form,category)",
                        "type": "string",
                    },
                    {"enum": [""]},
                ]
            },
            "name": {
                "description": "The name of a variable (i.e., field) as it appears in the data.\n",
                "title": "Variable Name",
                "type": "string",
            },
            "title": {
                "oneOf": [
                    {
                        "description": "The human-readable title of the variable.",
                        "title": "Variable Label (ie Title)",
                        "type": "string",
                    },
                    {"enum": [""]},
                ]
            },
            "description": {
                "description": "An extended description of the variable.",
                "title": "Variable Description",
                "type": "string",
            },
            "type": {
                "oneOf": [
                    {
                        "description": "A classification allowing the user (analyst, researcher or computer) to\nknow how to use the variable\n",
                        "title": "Variable Type",
                        "type": "string",
                        "enum": [
                            "boolean",
                            "any",
                            "date",
                            "yearmonth",
                            "string",
                            "integer",
                            "duration",
                            "number",
                            "geopoint",
                            "time",
                            "datetime",
                            "year",
                        ],
                    },
                    {"enum": [""]},
                ]
            },
            "format": {
                "oneOf": [
                    {
                        "description": "Indicates the format of the type specified in the `type` property. This\nmay describe the type of unit (such as for time fields like year or month)\nor the format of a date field (such as %y%m%d).\n",
                        "title": "Variable Format",
                        "type": "string",
                        "enum": [
                            "binary",
                            "email",
                            "any",
                            "uuid",
                            "object",
                            "uri",
                            "topojson",
                            "array",
                        ],
                    },
                    {"enum": [""]},
                ]
            },
            "constraints.maxLength": {
                "oneOf": [
                    {
                        "description": "Indicates the maximum length of an iterable (e.g., array, string, or\nobject). For example, if 'Hello World' is the longest value of a\ncategorical variable, this would be a maxLength of 11.\n",
                        "title": "Maximum Length",
                        "type": "integer",
                    },
                    {"enum": [""]},
                ]
            },
            "constraints.enum": {
                "oneOf": [
                    {
                        "description": "Constrains possible values to a set of values.",
                        "title": "Variable Possible Values",
                    },
                    {"enum": [""]},
                ]
            },
            "constraints.pattern": {
                "oneOf": [
                    {
                        "description": "A regular expression pattern the data MUST conform to.",
                        "title": "Regular Expression Pattern",
                        "type": "string",
                    },
                    {"enum": [""]},
                ]
            },
            "constraints.maximum": {
                "oneOf": [
                    {
                        "description": "Specifies the maximum value of a field (e.g., maximum -- or most\nrecent -- date, maximum integer etc). Note, this is different then\nmaxLength property.\n",
                        "title": "Maximum Value",
                        "type": "integer",
                    },
                    {"enum": [""]},
                ]
            },
            "encodings": {
                "oneOf": [
                    {
                        "description": 'Encodings (and mappings) allow categorical values to be stored as\nnumerical values. IMPORTANT: the ==key should be the value represented IN\nthe data== and the ==value should be the to-be-mapped label==. Many\nanalytic software programs use numerical encodings and some algorithms\nonly support numerical values. Additionally, this field provides a way to\nstore categoricals that are stored as  "short" labels (such as\nabbreviations)\n',
                        "title": "Variable Value Encodings (i.e., mappings; value labels)",
                    },
                    {"enum": [""]},
                ]
            },
            "ordered": {
                "oneOf": [
                    {
                        "description": "Indicates whether a categorical variable is ordered. This variable  is\nrelevant for variables that have an ordered relationship but not\nnecessarily  a numerical relationship (e.g., Strongly disagree < Disagree\n< Neutral < Agree).\n",
                        "title": "An ordered variable",
                    },
                    {"enum": [""]},
                ]
            },
            "missingValues": {
                "oneOf": [
                    {
                        "description": "A list of missing values specific to a variable.",
                        "title": "Missing Values",
                    },
                    {"enum": [""]},
                ]
            },
            "trueValues": {
                "oneOf": [
                    {
                        "description": "For boolean (true) variable (as defined in type field), this field allows\na physical string representation to be cast as true (increasing\nreadability of the field). It can include one or more values.\n",
                        "title": "Boolean True Value Labels",
                    },
                    {"enum": [""]},
                ]
            },
            "falseValues": {
                "oneOf": [
                    {
                        "description": "For boolean (false) variable (as defined in type field), this field allows\na physical string representation to be cast as false (increasing\nreadability of the field) that is not a standard false value. It can include one or more values.\n",
                        "title": "Boolean False Value Labels",
                    },
                    {"enum": [""]},
                ]
            },
            "repo_link": {
                "oneOf": [
                    {
                        "description": "A link to the variable as it exists on the home repository, if applicable\n",
                        "title": "Variable Repository Link",
                        "type": "string",
                    },
                    {"enum": [""]},
                ]
            },
            "cde_id": {
                "oneOf": [
                    {
                        "description": "The source and id for the NIH Common Data Elements program.",
                        "title": "Common Data Element Id",
                    },
                    {"enum": [""]},
                ]
            },
            "ontology_id": {
                "oneOf": [
                    {
                        "description": "Ontological information for the given variable as indicated  by the\nsource, id, and relation to the specified classification. One or more\nontology classifications can be specified. \n",
                        "title": "Ontology ID",
                    },
                    {"enum": [""]},
                ]
            },
            "univar_stats.median": {"oneOf": [{"type": "number"}, {"enum": [""]}]},
            "univar_stats.mean": {"oneOf": [{"type": "number"}, {"enum": [""]}]},
            "univar_stats.std": {"oneOf": [{"type": "number"}, {"enum": [""]}]},
            "univar_stats.min": {"oneOf": [{"type": "number"}, {"enum": [""]}]},
            "univar_stats.max": {"oneOf": [{"type": "number"}, {"enum": [""]}]},
            "univar_stats.mode": {"oneOf": [{"type": "number"}, {"enum": [""]}]},
            "univar_stats.count": {"oneOf": [{"type": "integer"}, {"enum": [""]}]},
            "univar_stats.twenty_five_percentile": {
                "oneOf": [{"type": "number"}, {"enum": [""]}]
            },
            "univar_stats.seventy_five_percentile": {
                "oneOf": [{"type": "number"}, {"enum": [""]}]
            },
            "univar_stats.cat_marginals": {"oneOf": [{"type": "array"}, {"enum": [""]}]},
        },
        "description": "Variable level metadata individual fields integrated into the variable level\nmetadata object within the HEAL platform metadata service.\n",
        "title": "HEAL Variable Level Metadata Fields",
    }


from pathlib import Path
import json
# from frictionless import Resource,Package
from collections.abc import MutableMapping
from .mappings import join_prop
from healdata_utils.utils import flatten_except_if
from os import PathLike

def convert_templatejson(
    jsontemplate:str,
    fields_name:str='data_dictionary',
    sep_iter = '|',
    sep_dict = '=',
    **kwargs
    ):
    """
    Converts a JSON file or dictionary conforming to HEAL specifications
    into a HEAL-specified data dictionary in both csv format and json format.

    Converts in-memory data or a path to a data dictionary file.
    
    Parameters
    ----------
    csvtemplate : str or path-like or an object that can be inferred as data by frictionless's Resource class.
        Data or path to data with the data being a tabular HEAL-specified data dictionary.
        This input can be any data object or path-like string excepted by a frictionless Resource object.
    data_dictionary_props : dict
        The HEAL-specified data dictionary properties.
    mappings : dict, optional
        Mappings (which can be a dictionary of either lambda functions or other to-be-mapped objects).
        Default: specified fieldmap.

    Returns
    -------
    dict
        A dictionary with two keys:
            - 'templatejson': the HEAL-specified JSON object.
            - 'templatecsv': the HEAL-specified tabular template.

    """
    if isinstance(jsontemplate,(str,PathLike)):
        data_dictionary_props = json.loads(Path(jsontemplate).read_text())
    elif isinstance(jsontemplate, MutableMapping):
        data_dictionary_props = jsontemplate
    else:
        raise Exception("jsontemplate needs to be either dictionary-like or a path to a json")


    fields_json = data_dictionary_props.pop(fields_name)
    fields_csv = []
    for f in fields_json:
        field_flattened = flatten_except_if(f)
        field_csv = {
            propname:join_prop(propname,prop)
            for propname,prop in field_flattened.items()
        }
        fields_csv.append(field_csv)

    template_json = dict(**data_dictionary_props,data_dictionary=fields_json)
    template_csv = dict(**data_dictionary_props,data_dictionary=fields_csv)

    return {"templatejson":template_json,"templatecsv":template_csv}


import os
from pathlib import Path
from healdata_utils.cli import convert_to_vlmd


# test convert vlmd with redcap csv
def test_convert_to_vlmd_with_redcap_csv_no_output():
    data_dictionary_metadata = {
        "description": (
            "This is a proof of concept to demonstrate"
            " the healdata-utils functionality"
        ),
        "title": "Healdata-utils Demonstration Data Dictionary",
    }
    filepath = "data/example_redcap_demo.redcap.csv"
    data_dictionaries = convert_to_vlmd(
        filepath, data_dictionary_props=data_dictionary_metadata
    )
    csvtemplate = data_dictionaries["csvtemplate"]
    jsontemplate = data_dictionaries["jsontemplate"]
    errors = data_dictionaries["errors"]

    assert csvtemplate[5] == {
        "module": "demographics",
        "name": "telephone_1",
        "title": "Phone number",
        "description": "Contact Information: Phone number",
        "type": "string",
        "format": "",
        "constraints.maxLength": "",
        "constraints.enum": "",
        "constraints.pattern": "^[0-9]{3}-[0-9]{3}-[0-9]{4}$",
        "constraints.maximum": "",
        "encodings": "",
        "ordered": "",
        "missingValues": "",
        "trueValues": "",
        "falseValues": "",
        "repo_link": "",
        "cde_id": "",
        "ontology_id": "",
        "univar_stats.median": "",
        "univar_stats.mean": "",
        "univar_stats.std": "",
        "univar_stats.min": "",
        "univar_stats.max": "",
        "univar_stats.mode": "",
        "univar_stats.count": "",
        "univar_stats.twenty_five_percentile": "",
        "univar_stats.seventy_five_percentile": "",
        "univar_stats.cat_marginals": "",
    }
    assert csvtemplate[7] == {
        "module": "demographics",
        "name": "email",
        "title": "E-mail",
        "description": "Contact Information: E-mail",
        "type": "string",
        "format": "email",
        "constraints.maxLength": "",
        "constraints.enum": "",
        "constraints.pattern": "",
        "constraints.maximum": "",
        "encodings": "",
        "ordered": "",
        "missingValues": "",
        "trueValues": "",
        "falseValues": "",
        "repo_link": "",
        "cde_id": "",
        "ontology_id": "",
        "univar_stats.median": "",
        "univar_stats.mean": "",
        "univar_stats.std": "",
        "univar_stats.min": "",
        "univar_stats.max": "",
        "univar_stats.mode": "",
        "univar_stats.count": "",
        "univar_stats.twenty_five_percentile": "",
        "univar_stats.seventy_five_percentile": "",
        "univar_stats.cat_marginals": "",
    }
    assert csvtemplate[8] == {
        "module": "demographics",
        "name": "sex",
        "title": "Gender",
        "description": "Contact Information: Gender",
        "type": "integer",
        "format": "",
        "constraints.maxLength": "",
        "constraints.enum": "0|1",
        "constraints.pattern": "",
        "constraints.maximum": "",
        "encodings": "0=Female|1=Male",
        "ordered": "",
        "missingValues": "",
        "trueValues": "",
        "falseValues": "",
        "repo_link": "",
        "cde_id": "",
        "ontology_id": "",
        "univar_stats.median": "",
        "univar_stats.mean": "",
        "univar_stats.std": "",
        "univar_stats.min": "",
        "univar_stats.max": "",
        "univar_stats.mode": "",
        "univar_stats.count": "",
        "univar_stats.twenty_five_percentile": "",
        "univar_stats.seventy_five_percentile": "",
        "univar_stats.cat_marginals": "",
    }

    assert jsontemplate["description"] == data_dictionary_metadata["description"]
    assert jsontemplate["title"] == data_dictionary_metadata["title"]
    assert jsontemplate["data_dictionary"][5] == {
        "name": "telephone_1",
        "type": "string",
        "constraints": {"pattern": "^[0-9]{3}-[0-9]{3}-[0-9]{4}$"},
        "description": "Contact Information: Phone number",
        "title": "Phone number",
        "module": "demographics",
    }
    assert jsontemplate["data_dictionary"][7] == {
        "name": "email",
        "type": "string",
        "format": "email",
        "description": "Contact Information: E-mail",
        "title": "E-mail",
        "module": "demographics",
    }
    assert jsontemplate["data_dictionary"][8] == {
        "name": "sex",
        "type": "integer",
        "encodings": {"0": "Female", "1": "Male"},
        "constraints": {"enum": ["0", "1"]},
        "description": "Contact Information: Gender",
        "title": "Gender",
        "module": "demographics",
    }

    assert errors["jsontemplate"] == {"valid": True, "errors": []}
    assert errors["csvtemplate"] == {"valid": True, "errors": []}


os.chdir(Path(__file__).parent)
test_convert_to_vlmd_with_redcap_csv_no_output()
print("SUCCESS")



from setuptools import setup, find_namespace_packages
from pathlib import Path

def generate_long_description():
    return Path("README.md").read_text()


setup(
    name='healdata_utils',
    version='0.1.1-alpha',
    author='Michael Kranz',
    author_email='kranz-michael@norc.org',
    long_description=generate_long_description(),
    long_description_content_type="text/markdown",    
    description='Data packaging tools for the HEAL data ecosystem',
    #TODO: change url to HEAL once migrated.
    url='https://github.com/norc-heal/healdata-utils',
    package_dir={'': 'src'},
    packages=find_namespace_packages(where='src'),
    install_requires=[
        'petl==1.7.12',
        'jsonschema==4.17.3',
        'requests==2.28.2',
        'PyYaml==6.0',
        #'frictionless==4.40.8',
        'pyreadstat==1.2.0',
        'charset_normalizer==2.1',
        'visions== 0.7.5',
        "click==8.1.3"
    ],
    entry_points='''
        [console_scripts]
        vlmd=healdata_utils.cli:main
    '''

)
